@@ -33,11 +33,10 @@
 
 #ifdef WIN32
 #define XCHAT_DIR "X-Chat 2"
-#define DEF_FONT "Monospace 10"
 #else
 #define XCHAT_DIR ".xchat2"
-#define DEF_FONT "Monospace 10"
 #endif
+#define DEF_FONT "Monospace 9"
 
 void
 list_addentry (GSList ** list, char *cmd, char *name)
@@ -62,29 +61,55 @@ list_addentry (GSList ** list, char *cmd, char *name)
 	*list = g_slist_append (*list, pop);
 }
 
+/* read it in from a buffer to our linked list */
+
+static void
+list_load_from_data (GSList ** list, char *ibuf, int size)
+{
+	char cmd[384];
+	char name[128];
+	char *buf;
+	int pnt = 0;
+
+	cmd[0] = 0;
+	name[0] = 0;
+
+	while (buf_get_line (ibuf, &buf, &pnt, size))
+	{
+		if (*buf != '#')
+		{
+			if (!strncasecmp (buf, "NAME ", 5))
+			{
+				safe_strcpy (name, buf + 5, sizeof (name));
+			}
+			else if (!strncasecmp (buf, "CMD ", 4))
+			{
+				safe_strcpy (cmd, buf + 4, sizeof (cmd));
+				if (*name)
+				{
+					list_addentry (list, cmd, name);
+					cmd[0] = 0;
+					name[0] = 0;
+				}
+			}
+		}
+	}
+}
+
 void
 list_loadconf (char *file, GSList ** list, char *defaultconf)
 {
 	char filebuf[256];
-	char cmd[256];
-	char name[82];
-	char *buf, *ibuf;
-	int fh, pnt = 0;
+	char *ibuf;
+	int fh;
 	struct stat st;
 
 	snprintf (filebuf, sizeof (filebuf), "%s/%s", get_xdir_fs (), file);
 	fh = open (filebuf, O_RDONLY | OFLAGS);
 	if (fh == -1)
 	{
-		if (!defaultconf)
-			return;
-		fh = open (filebuf, O_TRUNC | O_WRONLY | O_CREAT | OFLAGS, 0600);
-		if (fh != -1)
-		{
-			write (fh, defaultconf, strlen (defaultconf));
-			close (fh);
-			list_loadconf (file, list, defaultconf);
-		}
+		if (defaultconf)
+			list_load_from_data (list, defaultconf, strlen (defaultconf));
 		return;
 	}
 	if (fstat (fh, &st) != 0)
@@ -97,29 +122,8 @@ list_loadconf (char *file, GSList ** list, char *defaultconf)
 	read (fh, ibuf, st.st_size);
 	close (fh);
 
-	cmd[0] = 0;
-	name[0] = 0;
+	list_load_from_data (list, ibuf, st.st_size);
 
-	while (buf_get_line (ibuf, &buf, &pnt, st.st_size))
-	{
-		if (*buf != '#')
-		{
-			if (!strncasecmp (buf, "NAME ", 5))
-			{
-				safe_strcpy (name, buf + 5, sizeof (name));
-			}
-			if (!strncasecmp (buf, "CMD ", 4))
-			{
-				safe_strcpy (cmd, buf + 4, sizeof (cmd));
-				if (*name)
-				{
-					list_addentry (list, cmd, name);
-					cmd[0] = 0;
-					name[0] = 0;
-				}
-			}
-		}
-	}
 	free (ibuf);
 }
 
@@ -202,6 +206,17 @@ cfg_put_str (int fh, char *var, char *value)
 	return (write (fh, buf, len) == len);
 }
 
+int
+cfg_put_color (int fh, int r, int g, int b, char *var)
+{
+	char buf[400];
+	int len;
+
+	snprintf (buf, sizeof buf, "%s = %04x %04x %04x\n", var, r, g, b);
+	len = strlen (buf);
+	return (write (fh, buf, len) == len);
+}
+
 int
 cfg_put_int (int fh, int value, char *var)
 {
@@ -216,6 +231,18 @@ cfg_put_int (int fh, int value, char *var)
 	return (write (fh, buf, len) == len);
 }
 
+int
+cfg_get_color (char *cfg, char *var, int *r, int *g, int *b)
+{
+	char str[128];
+
+	if (!cfg_get_str (cfg, var, str, sizeof (str)))
+		return 0;
+
+	sscanf (str, "%04x %04x %04x", r, g, b);
+	return 1;
+}
+
 int
 cfg_get_int_with_result (char *cfg, char *var, int *result)
 {
@@ -309,25 +336,6 @@ get_xdir_utf8 (void)
 	return xdir_utf;
 }
 
-int
-mkdir_utf8 (char *dir)
-{
-	int ret;
-
-	dir = g_filename_from_utf8 (dir, -1, 0, 0, 0);
-	if (!dir)
-		return -1;
-
-#ifdef WIN32
-	ret = mkdir (dir);
-#else
-	ret = mkdir (dir, S_IRUSR | S_IWUSR | S_IXUSR);
-#endif
-	g_free (dir);
-
-	return ret;
-}
-
 static void
 check_prefs_dir (void)
 {
@@ -368,6 +376,7 @@ const struct prefs vars[] = {
 	{"away_show_once", P_OFFINT (show_away_once), TYPE_BOOL},
 	{"away_size_max", P_OFFINT (away_size_max), TYPE_INT},
 	{"away_timeout", P_OFFINT (away_timeout), TYPE_INT},
+	{"away_track", P_OFFINT (away_track), TYPE_BOOL},
 
 	{"completion_amount", P_OFFINT (completion_amount), TYPE_INT},
 	{"completion_auto", P_OFFINT (nickcompletion), TYPE_BOOL},
@@ -443,6 +452,7 @@ const struct prefs vars[] = {
 	{"input_beep_msg", P_OFFINT (beepmsg), TYPE_BOOL},
 	{"input_command_char", P_OFFSET (cmdchar), TYPE_STR},
 	{"input_filter_beep", P_OFFINT (filterbeep), TYPE_BOOL},
+	{"input_flash_hilight", P_OFFINT (flash_hilight), TYPE_BOOL},
 	{"input_fudge_snotice", P_OFFINT (fudgeservernotice), TYPE_BOOL},
 	{"input_perc_ascii", P_OFFINT (perc_ascii), TYPE_BOOL},
 	{"input_perc_color", P_OFFINT (perc_color), TYPE_BOOL},
@@ -452,6 +462,8 @@ const struct prefs vars[] = {
 	{"irc_conf_mode", P_OFFINT (confmode), TYPE_BOOL},
 	{"irc_extra_hilight", P_OFFSET (bluestring), TYPE_STR},
 	{"irc_hide_version", P_OFFINT (hidever), TYPE_BOOL},
+	{"irc_id_ytext", P_OFFSET (irc_id_ytext), TYPE_STR},
+	{"irc_id_ntext", P_OFFSET (irc_id_ntext), TYPE_STR},
 	{"irc_invisible", P_OFFINT (invisible), TYPE_BOOL},
 	{"irc_logging", P_OFFINT (logging), TYPE_BOOL},
 	{"irc_logmask", P_OFFSET (logmask), TYPE_STR},
@@ -498,10 +510,11 @@ const struct prefs vars[] = {
 	{"tab_chans", P_OFFINT (tabchannels), TYPE_BOOL},
 	{"tab_dialogs", P_OFFINT (privmsgtab), TYPE_BOOL},
 	{"tab_dnd",  P_OFFINT (tab_dnd), TYPE_BOOL},
-	{"tab_new_to_front", P_OFFINT (newtabstofront), TYPE_BOOL},
+	{"tab_new_to_front", P_OFFINT (newtabstofront), TYPE_INT},
 	{"tab_notices", P_OFFINT (notices_tabs), TYPE_BOOL},
 	{"tab_position", P_OFFINT (tabs_position), TYPE_INT},
 	{"tab_server", P_OFFINT (use_server_tab), TYPE_BOOL},
+	{"tab_small", P_OFFINT (tab_small), TYPE_BOOL},
 	{"tab_sort", P_OFFINT (tab_sort), TYPE_BOOL},
 	{"tab_trunc", P_OFFINT (truncchans), TYPE_INT},
 	{"tab_utils", P_OFFINT (windows_as_tabs), TYPE_BOOL},
@@ -571,11 +584,11 @@ load_config (void)
 	prefs.completion_amount = 5;
 	prefs.away_timeout = 60;
 	prefs.away_size_max = 300;
+	prefs.away_track = 1;
 	prefs.timestamp_logs = 1;
 	prefs.truncchans = 20;
 	prefs.autoresume = 1;
 	prefs.show_away_once = 1;
-	/*prefs.show_away_message = 1;*/
 	prefs.indent_nicks = 1;
 	prefs.thin_separator = 1;
 	/*prefs.tabs_position = 1;*/ /* 0 = bottom */
@@ -586,13 +599,12 @@ load_config (void)
 	prefs.autoreconnect = 1;
 	prefs.recon_delay = 10;
 	prefs.tabchannels = 1;
+	prefs.tab_sort = 1;
 	prefs.paned_userlist = 1;
-	prefs.newtabstofront = 1;
+	prefs.newtabstofront = 2;
 	prefs.use_server_tab = 1;
-	/*prefs.windows_as_tabs = 1;*/
 	prefs.privmsgtab = 1;
-	prefs.style_inputbox = 1;
-	/*prefs.slist_select = 99;*/
+	/*prefs.style_inputbox = 1;*/
 	prefs.nu_color = 4;
 	prefs.dccpermissions = 0600;
 	prefs.max_lines = 300;
@@ -625,6 +637,7 @@ load_config (void)
 	prefs.dcc_send_fillspaces = 1;
 	prefs.mainwindow_save = 1;
 	prefs.bantype = 2;
+	prefs.flash_hilight = 1;
 #ifdef WIN32
 	prefs.identd = 1;
 #endif
@@ -642,10 +655,15 @@ load_config (void)
 	strcpy (prefs.username, username);
 #ifdef WIN32
 	strcpy (prefs.sounddir, "./sounds");
-	if (strcmp (get_xdir_utf8 (), "./config") != 0)
-		sprintf (prefs.dccdir, "%s\\downloads", get_xdir_utf8 ());
-	else
-		strcpy (prefs.dccdir, "./downloads");
+	{
+		char out[256];
+
+		if (get_reg_str ("Software\\Microsoft\\Windows\\CurrentVersion\\"
+						 "Explorer\\Shell Folders", "Personal", out, sizeof (out)))
+			snprintf (prefs.dccdir, sizeof (prefs.dccdir), "%s\\Downloads", out);
+		else
+			snprintf (prefs.dccdir, sizeof (prefs.dccdir), "%s\\Downloads", get_xdir_utf8 ());
+	}
 #else
 	sprintf (prefs.sounddir, "%s/sounds", get_xdir_utf8 ());
 	sprintf (prefs.dccdir, "%s/downloads", get_xdir_utf8 ());
@@ -655,7 +673,6 @@ load_config (void)
 	strcpy (prefs.quitreason, _("Leaving"));
 	strcpy (prefs.partreason, prefs.quitreason);
 	strcpy (prefs.font_normal, DEF_FONT);
-	strcpy (prefs.soundcmd, "esdplay");
 	strcpy (prefs.dnsprogram, "host");
 
 	g_free ((char *)username);
@@ -857,17 +874,26 @@ cmd_set (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
 	int wild = FALSE;
 	int quiet = FALSE;
+	int erase = FALSE;
 	int i = 0, finds = 0, found;
-	char *var = word[2];
-	char *val = word_eol[3];
+	int idx = 2;
+	char *var, *val;
 
 	if (strcasecmp (word[2], "-quiet") == 0)
 	{
-		var = word[3];
-		val = word_eol[4];
+		idx++;
 		quiet = TRUE;
 	}
 
+	if (strcasecmp (word[2], "-e") == 0)
+	{
+		idx++;
+		erase = TRUE;
+	}
+
+	var = word[idx];
+	val = word_eol[idx+1];
+
 	if (!*var)
 	{
 		set_list (sess, tbuf);
@@ -893,7 +919,7 @@ cmd_set (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 			switch (vars[i].type)
 			{
 			case TYPE_STR:
-				if (*val)
+				if (erase || *val)
 				{
 					strncpy ((char *) &prefs + vars[i].offset, val, vars[i].len);
 					((char *) &prefs)[vars[i].offset + vars[i].len - 1] = 0;
