@@ -273,6 +273,29 @@ cmd_allchannels (session *sess, char *tbuf, char *word[], char *word_eol[])
 	return TRUE;
 }
 
+static int
+cmd_allchannelslocal (session *sess, char *tbuf, char *word[], char *word_eol[])
+{
+	GSList *list = sess_list;
+	server *serv = sess->server;
+
+	if (!*word_eol[2])
+		return FALSE;
+
+	while (list)
+	{
+		sess = list->data;
+		if (sess->type == SESS_CHANNEL && sess->channel[0] &&
+			 sess->server->connected && sess->server == serv)
+		{
+			handle_command (sess, word_eol[2], FALSE);
+		}
+		list = list->next;
+	}
+
+	return TRUE;
+}
+
 static int
 cmd_allservers (struct session *sess, char *tbuf, char *word[],
 					 char *word_eol[])
@@ -511,8 +534,12 @@ cmd_charset (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
 	server *serv = sess->server;
 	const char *locale = NULL;
+	int offset = 0;
 
-	if (!word[2][0])
+	if (strcmp (word[2], "-quiet") == 0)
+		offset++;
+
+	if (!word[2 + offset][0])
 	{
 		g_get_charset (&locale);
 		PrintTextf (sess, "Current charset: %s\n",
@@ -520,15 +547,16 @@ cmd_charset (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 		return TRUE;
 	}
 
-	if (servlist_check_encoding (word[2]))
+	if (servlist_check_encoding (word[2 + offset]))
 	{
 		if (serv->encoding)
 			free (serv->encoding);
-		serv->encoding = strdup (word[2]);
-		PrintTextf (sess, "Charset changed to: %s\n", word[2]);
+		serv->encoding = strdup (word[2 + offset]);
+		if (offset < 1)
+			PrintTextf (sess, "Charset changed to: %s\n", word[2 + offset]);
 	} else
 	{
-		PrintTextf (sess, "\0034Unknown charset:\017 %s\n", word[2]);
+		PrintTextf (sess, "\0034Unknown charset:\017 %s\n", word[2 + offset]);
 	}
 
 	return TRUE;
@@ -621,6 +649,20 @@ cmd_country (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	char *code = word[2];
 	if (*code)
 	{
+		/* search? */
+		if (strcmp (code, "-s") == 0)
+		{
+			country_search (word[3], sess, (void *)PrintTextf);
+			return TRUE;
+		}
+
+		/* search, but forgot the -s */
+		if (strchr (code, '*'))
+		{
+			country_search (code, sess, (void *)PrintTextf);
+			return TRUE;
+		}
+
 		sprintf (tbuf, "%s = %s\n", code, country (code));
 		PrintText (sess, tbuf);
 		return TRUE;
@@ -972,7 +1014,6 @@ static int
 cmd_discon (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
 	sess->server->disconnect (sess, TRUE, -1);
-	sess->server->network = NULL;
 	return TRUE;
 }
 
@@ -1324,6 +1365,7 @@ cmd_exec (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	int fds[2], pid = 0;
 	struct nbexec *s;
 	int shell = TRUE;
+	int fd;
 
 	if (*cmd)
 	{
@@ -1393,10 +1435,12 @@ cmd_exec (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 			dup2 (fds[1], 2);
 			/* Also copy it to stdin so we can write to it */
 			dup2 (fds[1], 0);
+			/* Now close all open file descriptors except stdin, stdout and stderr */
+			for (fd = 3; fd < 1024; fd++) close(fd);
 			/* Now we call /bin/sh to run our cmd ; made it more friendly -DC1 */
 			if (shell)
 			{
-				execl ("/bin/sh", "sh", "-c", cmd, 0);
+				execl ("/bin/sh", "sh", "-c", cmd, NULL);
 			} else
 			{
 				char **argv;
@@ -1566,6 +1610,12 @@ cmd_gui (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	} else if (!strcasecmp (word[2], "ICONIFY"))
 	{
 		fe_ctrl_gui (sess, 5, 0);
+	} else if (!strcasecmp (word[2], "MENU"))
+	{
+		if (!strcasecmp (word[3], "TOGGLE"))
+			fe_ctrl_gui (sess, 6, 0);
+		else
+			return FALSE;
 	} else
 	{
 		return FALSE;
@@ -1825,7 +1875,7 @@ lastlog (session *sess, char *search)
 
 	lastlog_sess = find_dialog (sess->server, "(lastlog)");
 	if (!lastlog_sess)
-		lastlog_sess = new_ircwindow (sess->server, "(lastlog)", SESS_DIALOG);
+		lastlog_sess = new_ircwindow (sess->server, "(lastlog)", SESS_DIALOG, 0);
 	lastlog_sess->lastlog_sess = sess;
 
 	fe_text_clear (lastlog_sess);
@@ -2036,7 +2086,7 @@ cmd_msg (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 				newsess = find_channel (sess->server, nick);
 			if (newsess)
 				inbound_chanmsg (newsess->server, NULL, newsess->channel,
-									  newsess->server->nick, msg, TRUE);
+									  newsess->server->nick, msg, TRUE, FALSE);
 			else
 			{
 				/* mask out passwords */
@@ -2077,7 +2127,7 @@ static int
 cmd_newserver (struct session *sess, char *tbuf, char *word[],
 					char *word_eol[])
 {
-	sess = new_ircwindow (NULL, NULL, SESS_SERVER);
+	sess = new_ircwindow (NULL, NULL, SESS_SERVER, 0);
 	cmd_server (sess, tbuf, word, word_eol);
 	return TRUE;
 }
@@ -2191,7 +2241,7 @@ cmd_query (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	if (*nick && !is_channel (sess->server, nick))
 	{
 		if (!find_dialog (sess->server, nick))
-			new_ircwindow (sess->server, nick, SESS_DIALOG);
+			new_ircwindow (sess->server, nick, SESS_DIALOG, 1);
 		return TRUE;
 	}
 	return FALSE;
@@ -2282,6 +2332,22 @@ cmd_say (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	return FALSE;
 }
 
+static int
+cmd_setcursor (struct session *sess, char *tbuf, char *word[], char *word_eol[])
+{
+	int delta = FALSE;
+
+	if (*word[2])
+	{
+		if (word[2][0] == '-' || word[2][0] == '+')
+			delta = TRUE;
+		fe_set_inputbox_cursor (sess, delta, atoi (word[2]));
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 static int
 cmd_settab (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
@@ -2296,6 +2362,18 @@ cmd_settab (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	return TRUE;
 }
 
+static int
+cmd_settext (struct session *sess, char *tbuf, char *word[], char *word_eol[])
+{
+	if (*word_eol[2])
+	{
+		fe_set_inputbox_contents (sess, word_eol[2]);
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 static int
 cmd_server (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
@@ -2323,6 +2401,8 @@ cmd_server (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	if (!(*server_name))
 		return FALSE;
 
+	sess->server->network = NULL;
+
 	/* dont clear it for /servchan */
 	if (strncasecmp (word_eol[1], "SERVCHAN ", 9))
 		sess->willjoinchannel[0] = 0;
@@ -2483,6 +2563,18 @@ cmd_unload (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	return FALSE;
 }
 
+static int
+cmd_url (struct session *sess, char *tbuf, char *word[], char *word_eol[])
+{
+	if (word[2][0])
+	{
+		fe_open_url (word[2]);
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 static int
 userlist_cb (struct User *user, session *sess)
 {
@@ -2571,7 +2663,7 @@ cmd_wallchan (struct session *sess, char *tbuf, char *word[],
 			if (sess->type == SESS_CHANNEL)
 			{
 				inbound_chanmsg (sess->server, NULL, sess->channel,
-									  sess->server->nick, word_eol[2], TRUE);
+									  sess->server->nick, word_eol[2], TRUE, FALSE);
 				sess->server->p_message (sess->server, sess->channel, word_eol[2]);
 			}
 			list = list->next;
@@ -2623,6 +2715,8 @@ const struct commands xc_cmds[] = {
 	 N_("ADDBUTTON <name> <action>, adds a button under the user-list")},
 	{"ALLCHAN", cmd_allchannels, 0, 0,
 	 N_("ALLCHAN <cmd>, sends a command to all channels you're in")},
+	{"ALLCHANL", cmd_allchannelslocal, 0, 0,
+	 N_("ALLCHANL <cmd>, sends a command to all channels you're in")},
 	{"ALLSERV", cmd_allservers, 0, 0,
 	 N_("ALLSERV <cmd>, sends a command to all servers you're in")},
 	{"AWAY", cmd_away, 1, 0, N_("AWAY [<reason>], sets you away")},
@@ -2633,7 +2727,7 @@ const struct commands xc_cmds[] = {
 	{"CLOSE", cmd_close, 0, 0, N_("CLOSE, Closes the current window/tab")},
 
 	{"COUNTRY", cmd_country, 0, 0,
-	 N_("COUNTRY <code>, finds a country code, eg: au = australia")},
+	 N_("COUNTRY [-s] <code|wildcard>, finds a country code, eg: au = australia")},
 	{"CTCP", cmd_ctcp, 1, 0,
 	 N_("CTCP <nick> <message>, send the CTCP message to nick, common messages are VERSION and USERINFO")},
 	{"CYCLE", cmd_cycle, 1, 1,
@@ -2678,7 +2772,7 @@ const struct commands xc_cmds[] = {
 	 N_("GATE <host> [<port>], proxies through a host, port defaults to 23")},
 	{"GETINT", cmd_getint, 0, 0, "GETINT <default> <command> <prompt>"},
 	{"GETSTR", cmd_getstr, 0, 0, "GETSTR <default> <command> <prompt>"},
-	{"GUI", cmd_gui, 0, 0, "GUI [SHOW|HIDE|FOCUS|FLASH|ICONIFY|COLOR <n>]"},
+	{"GUI", cmd_gui, 0, 0, "GUI [SHOW|HIDE|FOCUS|FLASH|ICONIFY|MENU TOGGLE|COLOR <n>]"},
 	{"HELP", cmd_help, 0, 0, 0},
 	{"HOP", cmd_hop, 1, 1,
 	 N_("HOP <nick>, gives chanhalf-op status to the nick (needs chanop)")},
@@ -2766,13 +2860,16 @@ const struct commands xc_cmds[] = {
 	 N_("SERVER <host> [<port>] [<password>], connects to a server, the default port is 6667")},
 #endif
 	{"SET", cmd_set, 0, 0, N_("SET [-quiet] <variable> [<value>]")},
+	{"SETCURSOR", cmd_setcursor, 0, 0, N_("SETCURSOR [-|+]<position>")},
 	{"SETTAB", cmd_settab, 0, 0, 0},
+	{"SETTEXT", cmd_settext, 0, 0, 0},
 	{"TOPIC", cmd_topic, 1, 1,
 	 N_("TOPIC [<topic>], sets the topic if one is given, else shows the current topic")},
 	{"UNBAN", cmd_unban, 1, 1,
 	 N_("UNBAN <mask> [<mask>...], unbans the specified masks.")},
 	{"UNIGNORE", cmd_unignore, 0, 0, N_("UNIGNORE <mask> [QUIET]")},
 	{"UNLOAD", cmd_unload, 0, 0, N_("UNLOAD <name>, unloads a plugin or script")},
+	{"URL", cmd_url, 0, 0, N_("URL <url>, opens a URL in your browser")},
 	{"USERLIST", cmd_userlist, 1, 1, 0},
 	{"VOICE", cmd_voice, 1, 1,
 	 N_("VOICE <nick>, gives voice status to someone (needs chanop)")},
@@ -3221,7 +3318,7 @@ handle_say (session *sess, char *text, int check_spch)
 		if (dcc)
 		{
 			inbound_chanmsg (sess->server, NULL, sess->channel,
-								  sess->server->nick, text, TRUE);
+								  sess->server->nick, text, TRUE, FALSE);
 			set_topic (sess, net_ip (dcc->addr));
 			goto xit;
 		}
@@ -3265,7 +3362,7 @@ handle_say (session *sess, char *text, int check_spch)
 		}
 
 		inbound_chanmsg (sess->server, sess, sess->channel, sess->server->nick,
-							  text, TRUE);
+							  text, TRUE, FALSE);
 		sess->server->p_message (sess->server, sess->channel, text);
 
 		if (t)
