@@ -27,6 +27,7 @@
 #include "outbound.h"
 #include "cfgfiles.h"
 #include "ignore.h"
+#include "servlist.h"
 #include "modes.h"
 #include "notify.h"
 #include "text.h"
@@ -263,10 +264,10 @@ plugin_add (session *sess, char *filename, void *handle, void *init_func,
 		pl->xchat_list_time = xchat_list_time;
 		pl->xchat_gettext = xchat_gettext;
 		pl->xchat_send_modes = xchat_send_modes;
+		pl->xchat_strip = xchat_strip;
+		pl->xchat_free = xchat_free;
 
 		/* incase new plugins are loaded on older xchat */
-		pl->xchat_dummy6 = xchat_dummy;
-		pl->xchat_dummy5 = xchat_dummy;
 		pl->xchat_dummy4 = xchat_dummy;
 		pl->xchat_dummy3 = xchat_dummy;
 		pl->xchat_dummy2 = xchat_dummy;
@@ -573,6 +574,34 @@ plugin_emit_dummy_print (session *sess, char *name)
 	return plugin_hook_run (sess, name, word, NULL, HOOK_PRINT);
 }
 
+int
+plugin_emit_keypress (session *sess, unsigned int state, unsigned int keyval,
+							 int len, char *string)
+{
+	char *word[PDIWORDS];
+	char keyval_str[16];
+	char state_str[16];
+	char len_str[16];
+	int i;
+
+	if (!hook_list)
+		return 0;
+
+	sprintf (keyval_str, "%u", keyval);
+	sprintf (state_str, "%u", state);
+	sprintf (len_str, "%d", len);
+
+	word[0] = "Key Press";
+	word[1] = keyval_str;
+	word[2] = state_str;
+	word[3] = string;
+	word[4] = len_str;
+	for (i = 5; i < PDIWORDS; i++)
+		word[i] = "\000";
+
+	return plugin_hook_run (sess, word[0], word, NULL, HOOK_PRINT);
+}
+
 static int
 plugin_timeout_cb (xchat_hook *hook)
 {
@@ -926,6 +955,18 @@ xchat_get_info (xchat_plugin *ph, const char *id)
   	case 0x2c0b7d03: /* channel */
 		return sess->channel;
 
+	case 0x2c0d614c: /* charset */
+		{
+			const char *locale;
+
+			if (sess->server->encoding)
+				return sess->server->encoding;
+
+			locale = NULL;
+			g_get_charset (&locale);
+			return locale;
+		}
+
 	case 0x30f5a8: /* host */
 		return sess->server->hostname;
 
@@ -941,6 +982,11 @@ xchat_get_info (xchat_plugin *ph, const char *id)
 	case 0x339763: /* nick */
 		return sess->server->nick;
 
+	case 0x438fdf9: /* nickserv */
+		if (sess->server->network)
+			return ((ircnet *)sess->server->network)->nickserv;
+		return NULL;
+
 	case 0xca022f43: /* server */
 		if (!sess->server->connected)
 			return NULL;
@@ -976,6 +1022,13 @@ xchat_get_prefs (xchat_plugin *ph, const char *name, const char **string, int *i
 {
 	int i = 0;
 
+	/* some special run-time info (not really prefs, but may aswell throw it in here) */
+	if (!strcmp (name, "state_cursor"))
+	{
+		*integer = fe_get_inputbox_cursor (ph->context);
+		return 2;
+	}
+
 	do
 	{
 		if (!strcasecmp (name, vars[i].name))
@@ -1083,33 +1136,33 @@ xchat_list_next (xchat_plugin *ph, xchat_list *xlist)
 	return 1;
 }
 
-const char **
+const char * const *
 xchat_list_fields (xchat_plugin *ph, const char *name)
 {
-	static const char *dcc_fields[] =
+	static const char * const dcc_fields[] =
 	{
 		"iaddress32","icps",		"sdestfile","sfile",		"snick",	"iport",
 		"ipos",		"iresume",	"isize",		"istatus", 	"itype",		NULL
 	};
-	static const char *channels_fields[] =
+	static const char * const channels_fields[] =
 	{
 		"schannel",	"schantypes", "pcontext", "iflags", "iid", "imaxmodes",
 		"snetwork", "snickmodes", "snickprefixes", "sserver", "itype", "iusers",
 		NULL
 	};
-	static const char *ignore_fields[] =
+	static const char * const ignore_fields[] =
 	{
 		"iflags", "smask", NULL
 	};
-	static const char *notify_fields[] =
+	static const char * const notify_fields[] =
 	{
 		"iflags", "snick", "toff", "ton", "tseen", NULL
 	};
-	static const char *users_fields[] =
+	static const char * const users_fields[] =
 	{
-		"iaway", "shost", "snick", "sprefix", NULL
+		"iaway", "shost", "tlasttalk", "snick", "sprefix", NULL
 	};
-	static const char *list_of_lists[] =
+	static const char * const list_of_lists[] =
 	{
 		"channels",	"dcc", "ignore", "notify", "users", NULL
 	};
@@ -1137,7 +1190,7 @@ time_t
 xchat_list_time (xchat_plugin *ph, xchat_list *xlist, const char *name)
 {
 	guint32 hash = str_hash (name);
-/*	gpointer data = xlist->pos->data;*/
+	gpointer data;
 
 	switch (xlist->type)
 	{
@@ -1153,6 +1206,15 @@ xchat_list_time (xchat_plugin *ph, xchat_list *xlist, const char *name)
 		case 0x35ce7b:	/* seen */
 			return xlist->notifyps->lastseen;
 		}
+		break;
+
+	case LIST_USERS:
+		data = xlist->pos->data;
+		switch (hash)
+		{
+		case 0xa9118c42:	/* lasttalk */
+			return ((struct User *)data)->lasttalk;
+		}
 	}
 
 	return (time_t) -1;
@@ -1379,3 +1441,15 @@ xchat_send_modes (xchat_plugin *ph, const char **targets, int ntargets, int mode
 
 	send_channel_modes (ph->context, tbuf, (char **)targets, 0, ntargets, sign, mode, modes_per_line);
 }
+
+char *
+xchat_strip (xchat_plugin *ph, const char *str, int len, int flags)
+{
+	return strip_color ((char *)str, len, (flags & 1), (flags & 2));
+}
+
+void
+xchat_free (xchat_plugin *ph, void *ptr)
+{
+	free (ptr);
+}
