@@ -45,15 +45,8 @@ static const struct defaultserver def[] =
 {
 	{"2600net",	0},
 	{0,			"irc.2600.net"},
-	{0,			"irc.dc2600.com"},
-	{0,			"irc.nyc2600.com"},
-
-	{"AbleNET",	0},
-	{0,			"california.ablenet.org"},
-	{0,			"amazon.ablenet.org"},
-	{0,			"agora.ablenet.org"},
-	{0,			"extreme.ablenet.org"},
-	{0,			"irc.ablenet.org"},
+	{0,			"ptolomea.2600.net"},
+	{0,			"collective.2600.net"},
 
 	{"AccessIRC",	0},
 	{0,			"irc.accessirc.net"},
@@ -85,12 +78,11 @@ static const struct defaultserver def[] =
 	{0,			"irc.if.lt"},
 
 	{"AmigaNet",	0},
-	{0,			"linux.us.amiganet.org"},
-	{0,			"whiterose.us.amiganet.org"},
-	{0,			"thule.no.amiganet.org"},
-	{0,			"dynarc.se.amiganet.org"},
-	{0,			"fullcomp.au.amiganet.org"},
-	{0,			"spod.uk.amiganet.org"},
+	{0,			"irc.amiganet.org"},
+	{0,			"us.amiganet.org"},
+	{0,			"uk.amiganet.org"},
+	{0,			"no.amiganet.org"},
+	{0,			"au.amiganet.org"},
 
 	{"ARCNet",	0},
 	{0,			"se1.arcnet.vapor.com"},
@@ -112,9 +104,9 @@ static const struct defaultserver def[] =
 	{0,			"irc.astrolink.org"},
 
 	{"AustNet",	0},
+	{0,			"au.austnet.org"},
 	{0,			"us.austnet.org"},
 	{0,			"ca.austnet.org"},
-	{0,			"au.austnet.org"},
 
 	{"AxeNet",	0},
 	{0,			"irc.axenet.org"},
@@ -137,6 +129,10 @@ static const struct defaultserver def[] =
 	{0,			"US.ChatNet.Org"},
 	{0,			"EU.ChatNet.Org"},
 
+	{"ChatSociety", 0},
+	{0,			"us.chatsociety.net"},
+	{0,			"eu.chatsociety.net"},
+
 	{"CoolChat",	0},
 	{0,			"irc.coolchat.net"},
 	{0,			"unix.coolchat.net"},
@@ -159,7 +155,6 @@ static const struct defaultserver def[] =
 	{0,			"irc.Prison.NET"},
 	{0,			"irc.Qeast.net"},
 	{0,			"irc.efnet.pl"},
-	{0,			"irc.flamed.net"},
 	{0,			"efnet.demon.co.uk"},
 	{0,			"irc.lagged.org"},
 	{0,			"irc.lightning.net"},
@@ -179,6 +174,9 @@ static const struct defaultserver def[] =
 	{0,			"irc.inn.at.euirc.net"},
 	{0,			"irc.bas.ch.euirc.net"},
 
+	{"EuropNet", 0},
+	{0,			"irc.europnet.org"},
+
 	{"EU-IRC",	0},
 	{0,			"irc.eu-irc.net"},
 
@@ -194,9 +192,6 @@ static const struct defaultserver def[] =
 
 	{"FreeNode",	0},
 	{0,				"irc.freenode.net"},
-	{0,				"irc.eu.freenode.net"},
-	{0,				"irc.au.freenode.net"},
-	{0,				"irc.us.freenode.net"},
 
 	{"Freeworld",	0},
 	{0,			"kabel.freeworld.nu"},
@@ -237,8 +232,9 @@ static const struct defaultserver def[] =
 	{0,			"Sunshine.Ca.US.Infinity-IRC.Org"},
 	{0,			"IRC.Infinity-IRC.Org"},
 
-	{"IRCDZone",		0},
-	{0,			"irc.ircdzone.net"},
+	{"insiderZ.DE",	0},
+	{0,			"irc.insiderz.de/6667"},
+	{0,			"irc.insiderz.de/6666"},
 
 	{"IrcLink",	0},
 	{0,			"irc.irclink.net"},
@@ -416,6 +412,9 @@ static const struct defaultserver def[] =
 	{0,			"moo.slashnet.org"},
 	{0,			"radon.slashnet.org"},
 
+	{"Sohbet.Net", 0},
+	{0,			"irc.sohbet.net"},
+
 	{"SorceryNet",	0},
 	{0,			"irc.sorcery.net/9000"},
 	{0,			"irc.us.sorcery.net/9000"},
@@ -497,7 +496,7 @@ servlist_connect (session *sess, ircnet *net)
 	server *serv;
 
 	if (!sess)
-		sess = new_ircwindow (NULL, NULL, SESS_SERVER);
+		sess = new_ircwindow (NULL, NULL, SESS_SERVER, 0);
 
 	serv = sess->server;
 
@@ -510,23 +509,15 @@ servlist_connect (session *sess, ircnet *net)
 	/* incase a protocol switch is added to the servlist gui */
 	server_fill_her_up (sess->server);
 
+	sess->willjoinchannel[0] = 0;
+	serv->password[0] = 0;
+
 	if (net->autojoin && !sess->server->recondelay_tag)
 		safe_strcpy (sess->willjoinchannel, net->autojoin,
 						 sizeof (sess->willjoinchannel));
 	if (net->pass)
 		safe_strcpy (serv->password, net->pass, sizeof (serv->password));
 
-	if (serv->username)
-	{
-		free (serv->username);
-		serv->username = NULL;
-	}
-	if (serv->realname)
-	{
-		free (serv->realname);
-		serv->realname = NULL;
-	}
-
 	if (net->flags & FLAG_USE_GLOBAL)
 	{
 		strcpy (serv->nick, prefs.nick1);
@@ -534,10 +525,6 @@ servlist_connect (session *sess, ircnet *net)
 	{
 		if (net->nick)
 			strcpy (serv->nick, net->nick);
-		if (net->user)
-			serv->username = strdup (net->user);
-		if (net->real)
-			serv->realname = strdup (net->real);
 	}
 
 	serv->dont_use_proxy = (net->flags & FLAG_USE_PROXY) ? FALSE : TRUE;
@@ -548,13 +535,7 @@ servlist_connect (session *sess, ircnet *net)
 		(net->flags & FLAG_ALLOW_INVALID) ? TRUE : FALSE;
 #endif
 
-	if (net->flags & FLAG_CYCLE)
-		/* needed later to do cycling */
-		serv->network = net;
-	else
-		serv->network = NULL;
-
-	serv->networkname = strdup (net->name);
+	serv->network = net;
 
 	port = strrchr (ircserv->hostname, '/');
 	if (port)
@@ -577,13 +558,6 @@ servlist_connect (session *sess, ircnet *net)
 	} else
 		serv->connect (serv, ircserv->hostname, 6667, FALSE);
 
-	if (net->command)
-	{
-		if (serv->eom_cmd)
-			free (serv->eom_cmd);
-		serv->eom_cmd = strdup (net->command);
-	}
-
 	if (serv->encoding)
 	{
 		free (serv->encoding);
@@ -685,9 +659,13 @@ servlist_cycle (server *serv)
 		max = g_slist_length (net->servlist);
 		if (max > 0)
 		{
-			net->selected++;
-			if (net->selected >= max)
-				net->selected = 0;
+			/* try the next server, if that option is on */
+			if (net->flags & FLAG_CYCLE)
+			{
+				net->selected++;
+				if (net->selected >= max)
+					net->selected = 0;
+			}
 
 			del = prefs.recon_delay * 1000;
 			if (del < 1000)
@@ -806,6 +784,8 @@ servlist_net_remove (ircnet *net)
 		free (net->autojoin);
 	if (net->command)
 		free (net->command);
+	if (net->nickserv)
+		free (net->nickserv);
 	if (net->comment)
 		free (net->comment);
 	if (net->encoding)
@@ -825,7 +805,7 @@ servlist_net_remove (ircnet *net)
 }
 
 ircnet *
-servlist_net_add (char *name, char *comment)
+servlist_net_add (char *name, char *comment, int prepend)
 {
 	ircnet *net;
 
@@ -835,7 +815,10 @@ servlist_net_add (char *name, char *comment)
 /*	net->comment = strdup (comment);*/
 	net->flags = FLAG_CYCLE | FLAG_USE_GLOBAL | FLAG_USE_PROXY;
 
-	network_list = g_slist_append (network_list, net);
+	if (prepend)
+		network_list = g_slist_prepend (network_list, net);
+	else
+		network_list = g_slist_append (network_list, net);
 
 	return net;
 }
@@ -850,7 +833,7 @@ servlist_load_defaults (void)
 	{
 		if (def[i].network)
 		{
-			net = servlist_net_add (def[i].network, def[i].host);
+			net = servlist_net_add (def[i].network, def[i].host, FALSE);
 #ifdef WIN32
 			/* Windows gets UTF-8 for new users. Unix gets "System Default",
 				which is often UTF-8 anyway! */
@@ -934,10 +917,13 @@ servlist_load (void)
 			case 'S':	/* new server/hostname for this network */
 				servlist_server_add (net, buf + 2);
 				break;
+			case 'B':
+				net->nickserv = strdup (buf + 2);
+				break;
 			}
 		}
 		if (buf[0] == 'N')
-			net = servlist_net_add (buf + 2, /* comment */ NULL);
+			net = servlist_net_add (buf + 2, /* comment */ NULL, FALSE);
 	}
 	fclose (fp);
 
@@ -1059,6 +1045,8 @@ servlist_save (void)
 			fprintf (fp, "P=%s\n", net->pass);
 		if (net->autojoin)
 			fprintf (fp, "J=%s\n", net->autojoin);
+		if (net->nickserv)
+			fprintf (fp, "B=%s\n", net->nickserv);
 		if (net->encoding && strcasecmp (net->encoding, "System") &&
 			 strcasecmp (net->encoding, "System default"))
 		{
