@@ -47,103 +47,6 @@ struct pevt_stage1
 };
 
 
-#ifdef USE_HEBREW
-
-static inline int iswhitespace(const char ch)
-{
-	return (ch == ' ' ||  ch == 8);
-}
-
-static inline int isheb(const char ch)
-{
-	return (((unsigned char)ch) >= 0xE0  &&  ((unsigned char)ch) <= 0xFA);
-}
-
-static inline int iseng(const char ch)
-{
-	return ((ch >= 'A'  &&  ch <= 'z')  ||  (ch >= 'a'  &&  ch <= 'z'));
-}
-
-static char *strhebpatch(char *dest, const char *src)
-{
-	short int mode = 0, imode;
-	const char *hmark = NULL, *lmark, *nmark, *nlmark;
-	char ch;
-
-	if (!dest || !src)
-		return NULL;
-	
-	for (;;)
-	{
-		if (mode == 0)
-		{
-			if (isheb(*src))
-			{
-				hmark = src;
-				mode = 1;
-			}
-			else
-				*dest++ = *src;
-		}
-		else if (mode == 1)
-		{
-			if (*src == 0  ||  iseng(*src))
-			{
-				lmark = src-1;
-				while (!isheb(*lmark)) lmark--;
-				src = lmark;
-				imode = 0;
-				nmark = NULL;
-
-				while (lmark >= hmark)
-				{
-					ch = *lmark;
-
-					if (imode == 0)
-						switch (ch)
-						{
-							case '(': ch = ')'; break;
-							case ')': ch = '('; break;
-							case '{': ch = '}'; break;
-							case '}': ch = '{'; break;
-							case '[': ch = ']'; break;
-							case ']': ch = '['; break;
-						}
-
-					if (imode == 0)
-					{
-						if (isdigit(ch))
-						{
-							imode = 1;
-							nmark = lmark;			
-						}
-						else 
-							*dest++ = ch;
-					}
-					else
-  						if (imode == 1  &&  (isheb(ch) || iswhitespace(ch)))
-						{
-							nlmark = lmark+1;
-							while (nlmark <= nmark)
-								*dest++ = *nlmark++;
-							imode = 0;
-							lmark++;
-						}
-					lmark--;
-				}
-				
-				hmark = NULL;
-				mode = 0;
-			}
-		}
-		if (!*src++)
-			break;
-	}
-	return dest;
-}
-
-#endif /* !USE_HEBREW */
-
 /* Make ip:port urls clikable (127.0.0.1:80 etc)
 *  patch by Alex <alex@cosinus.org> & dobler <dobler@barrysworld.com>
 */
@@ -330,8 +233,9 @@ log_create_filename (char *buf, char *servname, char *channame, char *netname)
 				*tmp = rfc_tolower (*tmp);
 				if (*tmp == '/')
 #else
-				/* win32 can't handle filenames with \|/ characters */
-				if (*tmp == '\\' || *tmp == '|' || *tmp == '/')
+				/* win32 can't handle filenames with \|/>< characters */
+				if (*tmp == '\\' || *tmp == '|' || *tmp == '/' ||
+					 *tmp == '>' || *tmp == '<')
 #endif
 					*tmp = '_';
 			}
@@ -431,7 +335,7 @@ log_open (session *sess)
 
 	log_close (sess);
 	sess->logfd = log_open_file (sess->server->servername, sess->channel,
-										  sess->server->networkname);
+										  get_network (sess, FALSE));
 
 	if (!log_error && sess->logfd == -1)
 	{
@@ -496,7 +400,7 @@ log_write (session *sess, char *text)
 				g_free (stamp);
 			}
 		}
-		temp = strip_color (text);
+		temp = strip_color (text, -1, 1, 1);
 		write (sess->logfd, temp, strlen (temp));
 		free (temp);
 	}
@@ -569,27 +473,7 @@ PrintText (session *sess, char *text)
 	}
 
 	log_write (sess, text);
-
-#ifdef USE_HEBREW
-	if (prefs.hebrew)
-	{
-   	const char *orgtext = NULL;
-   	if (text)
-   	{
-			orgtext = text;
-			text = strdup(text);
-			strhebpatch (text, orgtext);
-   	}
-		fe_print_text (sess, text);
-   	if (orgtext)
-			free(text);
-	} else
-	{
-#endif
-		fe_print_text (sess, text);
-#ifdef USE_HEBREW
-	}
-#endif
+	fe_print_text (sess, text);
 
 	if (conv)
 		g_free (conv);
@@ -671,484 +555,487 @@ char *pntevts[NUM_XP];
 
 #define pevt_generic_none_help NULL
 
-static char *pevt_genmsg_help[] = {
+static char * const pevt_genmsg_help[] = {
 	N_("Left message"),
 	N_("Right message"),
 };
 
-static char *pevt_join_help[] = {
+static char * const pevt_join_help[] = {
 	N_("The nick of the joining person"),
 	N_("The channel being joined"),
 	N_("The host of the person"),
 };
 
-static char *pevt_chanaction_help[] = {
+static char * const pevt_chanaction_help[] = {
 	N_("Nickname"),
 	N_("The action"),
 };
 
-static char *pevt_chanmsg_help[] = {
+static char * const pevt_chanmsg_help[] = {
 	N_("Nickname"),
 	N_("The text"),
 	N_("Mode char"),
+	N_("Identified text"),
 };
 
-static char *pevt_privmsg_help[] = {
+static char * const pevt_privmsg_help[] = {
 	N_("Nickname"),
 	N_("The message"),
 };
 
-static char *pevt_changenick_help[] = {
+static char * const pevt_changenick_help[] = {
 	N_("Old nickname"),
 	N_("New nickname"),
 };
 
-static char *pevt_newtopic_help[] = {
+static char * const pevt_newtopic_help[] = {
 	N_("Nick of person who changed the topic"),
 	N_("Topic"),
 	N_("Channel"),
 };
 
-static char *pevt_topic_help[] = {
+static char * const pevt_topic_help[] = {
 	N_("Channel"),
 	N_("Topic"),
 };
 
-static char *pevt_kick_help[] = {
+static char * const pevt_kick_help[] = {
 	N_("The nickname of the kicker"),
 	N_("The person being kicked"),
 	N_("The channel"),
 	N_("The reason"),
 };
 
-static char *pevt_part_help[] = {
+static char * const pevt_part_help[] = {
 	N_("The nick of the person leaving"),
 	N_("The host of the person"),
 	N_("The channel"),
 };
 
-static char *pevt_chandate_help[] = {
+static char * const pevt_chandate_help[] = {
 	N_("The channel"),
 	N_("The time"),
 };
 
-static char *pevt_topicdate_help[] = {
+static char * const pevt_topicdate_help[] = {
 	N_("The channel"),
 	N_("The creator"),
 	N_("The time"),
 };
 
-static char *pevt_quit_help[] = {
+static char * const pevt_quit_help[] = {
 	N_("Nick"),
 	N_("Reason"),
 	N_("Host"),
 };
 
-static char *pevt_pingrep_help[] = {
+static char * const pevt_pingrep_help[] = {
 	N_("Who it's from"),
 	N_("The time in x.x format (see below)"),
 };
 
-static char *pevt_notice_help[] = {
+static char * const pevt_notice_help[] = {
 	N_("Who it's from"),
 	N_("The message"),
 };
 
-static char *pevt_channotice_help[] = {
+static char * const pevt_channotice_help[] = {
 	N_("Who it's from"),
 	N_("The Channel it's going to"),
 	N_("The message"),
 };
 
-static char *pevt_dprivmsg_help[] = {
+static char * const pevt_dprivmsg_help[] = {
 	N_("Nickname"),
 	N_("The message"),
+	N_("Identified text"),
 };
 
-static char *pevt_uchangenick_help[] = {
+static char * const pevt_uchangenick_help[] = {
 	N_("Old nickname"),
 	N_("New nickname"),
 };
 
-static char *pevt_ukick_help[] = {
+static char * const pevt_ukick_help[] = {
 	N_("The person being kicked"),
 	N_("The channel"),
 	N_("The nickname of the kicker"),
 	N_("The reason"),
 };
 
-static char *pevt_partreason_help[] = {
+static char * const pevt_partreason_help[] = {
 	N_("The nick of the person leaving"),
 	N_("The host of the person"),
 	N_("The channel"),
 	N_("The reason"),
 };
 
-static char *pevt_ctcpsnd_help[] = {
+static char * const pevt_ctcpsnd_help[] = {
 	N_("The sound"),
 	N_("The nick of the person"),
 };
 
-static char *pevt_ctcpgen_help[] = {
+static char * const pevt_ctcpgen_help[] = {
 	N_("The CTCP event"),
 	N_("The nick of the person"),
 };
 
-static char *pevt_ctcpgenc_help[] = {
+static char * const pevt_ctcpgenc_help[] = {
 	N_("The CTCP event"),
 	N_("The nick of the person"),
 	N_("The Channel it's going to"),
 };
 
-static char *pevt_chansetkey_help[] = {
+static char * const pevt_chansetkey_help[] = {
 	N_("The nick of the person who set the key"),
 	N_("The key"),
 };
 
-static char *pevt_chansetlimit_help[] = {
+static char * const pevt_chansetlimit_help[] = {
 	N_("The nick of the person who set the limit"),
 	N_("The limit"),
 };
 
-static char *pevt_chanop_help[] = {
+static char * const pevt_chanop_help[] = {
 	N_("The nick of the person who did the op'ing"),
 	N_("The nick of the person who has been op'ed"),
 };
 
-static char *pevt_chanhop_help[] = {
+static char * const pevt_chanhop_help[] = {
 	N_("The nick of the person who has been halfop'ed"),
 	N_("The nick of the person who did the halfop'ing"),
 };
 
-static char *pevt_chanvoice_help[] = {
+static char * const pevt_chanvoice_help[] = {
 	N_("The nick of the person who did the voice'ing"),
 	N_("The nick of the person who has been voice'ed"),
 };
 
-static char *pevt_chanban_help[] = {
+static char * const pevt_chanban_help[] = {
 	N_("The nick of the person who did the banning"),
 	N_("The ban mask"),
 };
 
-static char *pevt_chanrmkey_help[] = {
+static char * const pevt_chanrmkey_help[] = {
 	N_("The nick who removed the key"),
 };
 
-static char *pevt_chanrmlimit_help[] = {
+static char * const pevt_chanrmlimit_help[] = {
 	N_("The nick who removed the limit"),
 };
 
-static char *pevt_chandeop_help[] = {
+static char * const pevt_chandeop_help[] = {
 	N_("The nick of the person of did the deop'ing"),
 	N_("The nick of the person who has been deop'ed"),
 };
-static char *pevt_chandehop_help[] = {
+static char * const pevt_chandehop_help[] = {
 	N_("The nick of the person of did the dehalfop'ing"),
 	N_("The nick of the person who has been dehalfop'ed"),
 };
 
-static char *pevt_chandevoice_help[] = {
+static char * const pevt_chandevoice_help[] = {
 	N_("The nick of the person of did the devoice'ing"),
 	N_("The nick of the person who has been devoice'ed"),
 };
 
-static char *pevt_chanunban_help[] = {
+static char * const pevt_chanunban_help[] = {
 	N_("The nick of the person of did the unban'ing"),
 	N_("The ban mask"),
 };
 
-static char *pevt_chanexempt_help[] = {
+static char * const pevt_chanexempt_help[] = {
 	N_("The nick of the person who did the exempt"),
 	N_("The exempt mask"),
 };
 
-static char *pevt_chanrmexempt_help[] = {
+static char * const pevt_chanrmexempt_help[] = {
 	N_("The nick of the person removed the exempt"),
 	N_("The exempt mask"),
 };
 
-static char *pevt_chaninvite_help[] = {
+static char * const pevt_chaninvite_help[] = {
 	N_("The nick of the person who did the invite"),
 	N_("The invite mask"),
 };
 
-static char *pevt_chanrminvite_help[] = {
+static char * const pevt_chanrminvite_help[] = {
 	N_("The nick of the person removed the invite"),
 	N_("The invite mask"),
 };
 
-static char *pevt_chanmodegen_help[] = {
+static char * const pevt_chanmodegen_help[] = {
 	N_("The nick of the person setting the mode"),
 	N_("The mode's sign (+/-)"),
 	N_("The mode letter"),
 	N_("The channel it's being set on"),
 };
 
-static char *pevt_whois1_help[] = {
+static char * const pevt_whois1_help[] = {
 	N_("Nickname"),
 	N_("Username"),
 	N_("Host"),
 	N_("Full name"),
 };
 
-static char *pevt_whois2_help[] = {
+static char * const pevt_whois2_help[] = {
 	N_("Nickname"),
 	N_("Channel Membership/\"is an IRC operator\""),
 };
 
-static char *pevt_whois3_help[] = {
+static char * const pevt_whois3_help[] = {
 	N_("Nickname"),
 	N_("Server Information"),
 };
 
-static char *pevt_whois4_help[] = {
+static char * const pevt_whois4_help[] = {
 	N_("Nickname"),
 	N_("Idle time"),
 };
 
-static char *pevt_whois4t_help[] = {
+static char * const pevt_whois4t_help[] = {
 	N_("Nickname"),
 	N_("Idle time"),
 	N_("Signon time"),
 };
 
-static char *pevt_whois5_help[] = {
+static char * const pevt_whois5_help[] = {
 	N_("Nickname"),
 	N_("Away reason"),
 };
 
-static char *pevt_whois6_help[] = {
+static char * const pevt_whois6_help[] = {
 	N_("Nickname"),
 };
 
-static char *pevt_whoisid_help[] = {
+static char * const pevt_whoisid_help[] = {
 	N_("Nickname"),
 	N_("Message"),
+	"Numeric"
 };
 
-static char *pevt_whoisauth_help[] = {
+static char * const pevt_whoisauth_help[] = {
 	N_("Nickname"),
 	N_("Message"),
 	N_("Account"),
 };
 
-static char *pevt_whoisrealhost_help[] = {
+static char * const pevt_whoisrealhost_help[] = {
 	N_("Nickname"),
 	N_("Real user@host"),
 	N_("Real IP"),
 	N_("Message"),
 };
 
-static char *pevt_generic_channel_help[] = {
+static char * const pevt_generic_channel_help[] = {
 	N_("Channel Name"),
 };
 
-static char *pevt_servertext_help[] = {
+static char * const pevt_servertext_help[] = {
 	N_("Text"),
 	N_("Server Name")
 };
 
-static char *pevt_invited_help[] = {
+static char * const pevt_invited_help[] = {
 	N_("Channel Name"),
 	N_("Nick of person who invited you"),
 	N_("Server Name"),
 };
 
-static char *pevt_usersonchan_help[] = {
+static char * const pevt_usersonchan_help[] = {
 	N_("Channel Name"),
 	N_("Users"),
 };
 
-static char *pevt_nickclash_help[] = {
+static char * const pevt_nickclash_help[] = {
 	N_("Nickname in use"),
 	N_("Nick being tried"),
 };
 
-static char *pevt_connfail_help[] = {
+static char * const pevt_connfail_help[] = {
 	N_("Error"),
 };
 
-static char *pevt_connect_help[] = {
+static char * const pevt_connect_help[] = {
 	N_("Host"),
 	N_("IP"),
 	N_("Port"),
 };
 
-static char *pevt_sconnect_help[] = {
+static char * const pevt_sconnect_help[] = {
 	N_("PID"),
 };
 
-static char *pevt_generic_nick_help[] = {
+static char * const pevt_generic_nick_help[] = {
 	N_("Nickname"),
 };
 
-static char *pevt_chanmodes_help[] = {
+static char * const pevt_chanmodes_help[] = {
 	N_("Channel name"),
 	N_("Modes string"),
 };
 
-static char *pevt_rawmodes_help[] = {
+static char * const pevt_rawmodes_help[] = {
 	N_("Nickname"),
 	N_("Modes string"),
 };
 
-static char *pevt_kill_help[] = {
+static char * const pevt_kill_help[] = {
 	N_("Nickname"),
 	N_("Reason"),
 };
 
-static char *pevt_dccchaterr_help[] = {
+static char * const pevt_dccchaterr_help[] = {
 	N_("Nickname"),
 	N_("IP address"),
 	N_("Port"),
 	N_("Error"),
 };
 
-static char *pevt_dccstall_help[] = {
+static char * const pevt_dccstall_help[] = {
 	N_("DCC Type"),
 	N_("Filename"),
 	N_("Nickname"),
 };
 
-static char *pevt_generic_file_help[] = {
+static char * const pevt_generic_file_help[] = {
 	N_("Filename"),
 	N_("Error"),
 };
 
-static char *pevt_dccrecverr_help[] = {
+static char * const pevt_dccrecverr_help[] = {
 	N_("Filename"),
 	N_("Destination filename"),
 	N_("Nickname"),
 	N_("Error"),
 };
 
-static char *pevt_dccrecvcomp_help[] = {
+static char * const pevt_dccrecvcomp_help[] = {
 	N_("Filename"),
 	N_("Destination filename"),
 	N_("Nickname"),
 	N_("CPS"),
 };
 
-static char *pevt_dccconfail_help[] = {
+static char * const pevt_dccconfail_help[] = {
 	N_("DCC Type"),
 	N_("Nickname"),
 	N_("Error"),
 };
 
-static char *pevt_dccchatcon_help[] = {
+static char * const pevt_dccchatcon_help[] = {
 	N_("Nickname"),
 	N_("IP address"),
 };
 
-static char *pevt_dcccon_help[] = {
+static char * const pevt_dcccon_help[] = {
 	N_("Nickname"),
 	N_("IP address"),
 	N_("Filename"),
 };
 
-static char *pevt_dccsendfail_help[] = {
+static char * const pevt_dccsendfail_help[] = {
 	N_("Filename"),
 	N_("Nickname"),
 	N_("Error"),
 };
 
-static char *pevt_dccsendcomp_help[] = {
+static char * const pevt_dccsendcomp_help[] = {
 	N_("Filename"),
 	N_("Nickname"),
 	N_("CPS"),
 };
 
-static char *pevt_dccoffer_help[] = {
+static char * const pevt_dccoffer_help[] = {
 	N_("Filename"),
 	N_("Nickname"),
 	N_("Pathname"),
 };
 
-static char *pevt_dccfileabort_help[] = {
+static char * const pevt_dccfileabort_help[] = {
 	N_("Nickname"),
 	N_("Filename")
 };
 
-static char *pevt_dccchatabort_help[] = {
+static char * const pevt_dccchatabort_help[] = {
 	N_("Nickname"),
 };
 
-static char *pevt_dccresumeoffer_help[] = {
+static char * const pevt_dccresumeoffer_help[] = {
 	N_("Nickname"),
 	N_("Filename"),
 	N_("Position"),
 };
 
-static char *pevt_dccsendoffer_help[] = {
+static char * const pevt_dccsendoffer_help[] = {
 	N_("Nickname"),
 	N_("Filename"),
 	N_("Size"),
 	N_("IP address"),
 };
 
-static char *pevt_dccgenericoffer_help[] = {
+static char * const pevt_dccgenericoffer_help[] = {
 	N_("DCC String"),
 	N_("Nickname"),
 };
 
-static char *pevt_notifynumber_help[] = {
+static char * const pevt_notifynumber_help[] = {
 	N_("Number of notify items"),
 };
 
-static char *pevt_serverlookup_help[] = {
+static char * const pevt_serverlookup_help[] = {
 	N_("Servername"),
 };
 
-static char *pevt_servererror_help[] = {
+static char * const pevt_servererror_help[] = {
 	N_("Text"),
 };
 
-static char *pevt_foundip_help[] = {
+static char * const pevt_foundip_help[] = {
 	N_("IP"),
 };
 
-static char *pevt_dccrename_help[] = {
+static char * const pevt_dccrename_help[] = {
 	N_("Old Filename"),
 	N_("New Filename"),
 };
 
-static char *pevt_ctcpsend_help[] = {
+static char * const pevt_ctcpsend_help[] = {
 	N_("Receiver"),
 	N_("Message"),
 };
 
-static char *pevt_ignoreaddremove_help[] = {
+static char * const pevt_ignoreaddremove_help[] = {
 	N_("Hostmask"),
 };
 
-static char *pevt_resolvinguser_help[] = {
+static char * const pevt_resolvinguser_help[] = {
 	N_("Nickname"),
 	N_("Hostname"),
 };
 
-static char *pevt_malformed_help[] = {
+static char * const pevt_malformed_help[] = {
 	N_("Nickname"),
 	N_("The Packet"),
 };
 
-static char *pevt_pingtimeout_help[] = {
+static char * const pevt_pingtimeout_help[] = {
 	N_("Seconds"),
 };
 
-static char *pevt_uinvite_help[] = {
+static char * const pevt_uinvite_help[] = {
 	N_("Nick of person who have been invited"),
 	N_("Channel Name"),
 	N_("Server Name"),
 };
 
-static char *pevt_banlist_help[] = {
+static char * const pevt_banlist_help[] = {
 	N_("Channel"),
 	N_("Banmask"),
 	N_("Who set the ban"),
 	N_("Ban time"),
 };
 
-static char *pevt_discon_help[] = {
+static char * const pevt_discon_help[] = {
 	N_("Error"),
 };
 
@@ -1157,16 +1044,18 @@ static char *pevt_discon_help[] = {
 static void
 pevent_load_defaults ()
 {
-	int i, len;
+	int i;
 
 	for (i = 0; i < NUM_XP; i++)
 	{
-		len = strlen (_(te[i].def));
-		len++;
 		if (pntevts_text[i])
 			free (pntevts_text[i]);
-		pntevts_text[i] = malloc (len);
-		memcpy (pntevts_text[i], _(te[i].def), len);
+
+		/* don't gettext() the blank ones */
+		if (i == XP_TE_OPENDIALOG || i == XP_TE_BEEP)
+			pntevts_text[i] = strdup (te[i].def);
+		else
+			pntevts_text[i] = strdup (_(te[i].def));
 	}
 }
 
@@ -1183,7 +1072,7 @@ pevent_make_pntevts ()
 		if (pevt_build_string (pntevts_text[i], &(pntevts[i]), &m) != 0)
 		{
 			snprintf (out, sizeof (out),
-						 _("Error parsing event %s.\nLoading default"), te[i].name);
+						 _("Error parsing event %s.\nLoading default."), te[i].name);
 			fe_message (out, FALSE);
 			free (pntevts_text[i]);
 			len = strlen (te[i].def) + 1;
@@ -1196,7 +1085,6 @@ pevent_make_pntevts ()
 				abort ();
 			}
 		}
-		check_special_chars (pntevts[i], TRUE);
 	}
 }
 
@@ -1221,15 +1109,6 @@ pevent_trigger_load (int *i_penum, char **i_text, char **i_snd)
 			free (pntevts_text[penum]);
 		pntevts_text[penum] = malloc (len);
 		memcpy (pntevts_text[penum], text, len);
-		if (te[penum].sound)
-			free (te[penum].sound);
-		if (snd)
-		{
-			len = strlen (snd) + 1;
-			te[penum].sound = malloc (len);
-			memcpy (te[penum].sound, snd, len);
-		} else
-			te[penum].sound = NULL;
 	}
 
 	if (text)
@@ -1318,6 +1197,7 @@ pevent_load (char *filename)
 			if (text)
 				free (text);
 
+#if 0
 			/* This allows updating of old strings. We don't use new defaults
 				if the user has customized the strings (.e.g a text theme).
 				Hash of the old default is enough to identify and replace it.
@@ -1343,15 +1223,18 @@ pevent_load (char *filename)
 			default:
 				text = strdup (ofs);
 			}
+#else
+			text = strdup (ofs);
+#endif
 
 			continue;
-		} else if (strcmp (buf, "event_sound") == 0)
+		}/* else if (strcmp (buf, "event_sound") == 0)
 		{
 			if (snd)
 				free (snd);
 			snd = strdup (ofs);
 			continue;
-		}
+		}*/
 
 		continue;
 	}
@@ -1644,62 +1527,6 @@ pevt_build_string (const char *input, char **output, int *max_arg)
 	return 0;
 }
 
-static void
-play_wave (const char *file)
-{
-	char buf[512];
-	char wavfile[512];
-	char *file_fs;
-
-	/* the pevents GUI editor triggers this after removing a soundfile */
-	if (!file[0])
-		return;
-
-	memset (buf, 0, sizeof (buf));
-	memset (wavfile, 0, sizeof (wavfile));
-
-#ifdef WIN32
-	/* check for fullpath, windows style */
-	if (strlen (file) > 3 &&
-		 file[1] == ':' && (file[2] == '\\' || file[2] == '/') )
-	{
-		strncpy (wavfile, file, sizeof (wavfile));
-	} else
-#endif
-	if (file[0] != '/')
-	{
-		snprintf (wavfile, sizeof (wavfile), "%s/%s", prefs.sounddir, file);
-	} else
-	{
-		strncpy (wavfile, file, sizeof (wavfile));
-	}
-
-	file_fs = g_filename_from_utf8 (wavfile, -1, 0, 0, 0);
-	if (!file_fs)
-		return;
-
-	if (access (file_fs, R_OK) == 0)
-	{
-#ifdef WIN32
-		if (strcmp (prefs.soundcmd, "esdplay") == 0)
-		{
-			PlaySound (file_fs, NULL, SND_NODEFAULT|SND_FILENAME|SND_ASYNC);
-		} else
-#endif
-		{
-			snprintf (buf, sizeof (buf), "%s %s", prefs.soundcmd, file_fs);
-			buf[sizeof (buf) - 1] = '\0';
-			xchat_exec (buf);
-		}
-	} else
-	{
-		snprintf (buf, sizeof (buf), _("Cannot read sound file:\n%s"), wavfile);
-		fe_message (buf, FALSE);
-	}
-
-	g_free (file_fs);
-}
-
 /* called by EMIT_SIGNAL macro */
 
 void
@@ -1728,8 +1555,7 @@ text_emit (int index, session *sess, char *a, char *b, char *c, char *d)
 	if (plugin_emit_print (sess, word))
 		return;
 
-	if (te[index].sound)
-		play_wave (te[index].sound);
+	sound_play_event (index);
 
 	display_event (pntevts[index], sess, te[index].num_args, word);
 }
@@ -1777,11 +1603,194 @@ pevent_save (char *fn)
 		write (fd, buf, snprintf (buf, sizeof (buf),
 										  "event_name=%s\n", te[i].name));
 		write (fd, buf, snprintf (buf, sizeof (buf),
-										  "event_text=%s\n", pntevts_text[i]));
-		if (te[i].sound && te[i].sound[0])
+										  "event_text=%s\n\n", pntevts_text[i]));
+	}
+
+	close (fd);
+}
+
+/* =========================== */
+/* ========== SOUND ========== */
+/* =========================== */
+
+char *sound_files[NUM_XP];
+
+void
+sound_beep (session *sess)
+{
+	if (sound_files[XP_TE_BEEP] && sound_files[XP_TE_BEEP][0])
+		/* user defined beep _file_ */
+		sound_play_event (XP_TE_BEEP);
+	else
+		/* system beep */
+		fe_beep ();
+}
+
+static char *
+sound_find_command (void)
+{
+	/* some sensible unix players. You're bound to have one of them */
+	static const char * const progs[] = {"aplay", "esdplay", "soxplay", "artsplay", NULL};
+	char *cmd;
+	int i = 0;
+
+	if (prefs.soundcmd[0])
+		return g_strdup (prefs.soundcmd);
+
+	while (progs[i])
+	{
+		cmd = g_find_program_in_path (progs[i]);
+		if (cmd)
+			return cmd;
+		i++;
+	}
+
+	return NULL;
+}
+
+void
+sound_play (const char *file)
+{
+	char buf[512];
+	char wavfile[512];
+	char *file_fs;
+	char *cmd;
+
+	/* the pevents GUI editor triggers this after removing a soundfile */
+	if (!file[0])
+		return;
+
+	memset (buf, 0, sizeof (buf));
+	memset (wavfile, 0, sizeof (wavfile));
+
+#ifdef WIN32
+	/* check for fullpath, windows style */
+	if (strlen (file) > 3 &&
+		 file[1] == ':' && (file[2] == '\\' || file[2] == '/') )
+	{
+		strncpy (wavfile, file, sizeof (wavfile));
+	} else
+#endif
+	if (file[0] != '/')
+	{
+		snprintf (wavfile, sizeof (wavfile), "%s/%s", prefs.sounddir, file);
+	} else
+	{
+		strncpy (wavfile, file, sizeof (wavfile));
+	}
+
+	file_fs = g_filename_from_utf8 (wavfile, -1, 0, 0, 0);
+	if (!file_fs)
+		return;
+
+	if (access (file_fs, R_OK) == 0)
+	{
+		cmd = sound_find_command ();
+
+#ifdef WIN32
+		if (cmd == NULL || strcmp (cmd, "esdplay") == 0)
+		{
+			PlaySound (file_fs, NULL, SND_NODEFAULT|SND_FILENAME|SND_ASYNC);
+		} else
+#endif
+		{
+			if (cmd)
+			{
+				if (strchr (file_fs, ' '))
+					snprintf (buf, sizeof (buf), "%s \"%s\"", cmd, file_fs);
+				else
+					snprintf (buf, sizeof (buf), "%s %s", cmd, file_fs);
+				buf[sizeof (buf) - 1] = '\0';
+				xchat_exec (buf);
+			}
+		}
+
+		if (cmd)
+			g_free (cmd);
+
+	} else
+	{
+		snprintf (buf, sizeof (buf), _("Cannot read sound file:\n%s"), wavfile);
+		fe_message (buf, FALSE);
+	}
+
+	g_free (file_fs);
+}
+
+void
+sound_play_event (int i)
+{
+	if (sound_files[i])
+		sound_play (sound_files[i]);
+}
+
+static void
+sound_load_event (char *evt, char *file)
+{
+	int i = 0;
+
+	if (file[0] && pevent_find (evt, &i) != -1)
+	{
+		if (sound_files[i])
+			free (sound_files[i]);
+		sound_files[i] = strdup (file);
+	}
+}
+
+void
+sound_load ()
+{
+	int fd;
+	char buf[512];
+	char evt[128];
+
+	memset (&sound_files, 0, sizeof (char *) * (NUM_XP));
+
+	snprintf (buf, sizeof (buf), "%s/sound.conf", get_xdir_fs ());
+	fd = open (buf, O_RDONLY | OFLAGS);
+	if (fd == -1)
+		return;
+
+	evt[0] = 0;
+	while (waitline (fd, buf, sizeof buf, FALSE) != -1)
+	{
+		if (strncmp (buf, "event=", 6) == 0)
+		{
+			safe_strcpy (evt, buf + 6, sizeof (evt));
+		}
+		else if (strncmp (buf, "sound=", 6) == 0)
+		{
+			if (evt[0] != 0)
+			{
+				sound_load_event (evt, buf + 6);
+				evt[0] = 0;
+			}
+		}
+	}
+
+	close (fd);
+}
+
+void
+sound_save ()
+{
+	int fd, i;
+	char buf[512];
+
+	snprintf (buf, sizeof (buf), "%s/sound.conf", get_xdir_fs ());
+	fd = open (buf, O_CREAT | O_TRUNC | O_WRONLY | OFLAGS, 0x180);
+	if (fd == -1)
+		return;
+
+	for (i = 0; i < NUM_XP; i++)
+	{
+		if (sound_files[i] && sound_files[i][0])
+		{
 			write (fd, buf, snprintf (buf, sizeof (buf),
-											  "event_sound=%s\n", te[i].sound));
-		write (fd, "\n", 1);
+											  "event=%s\n", te[i].name));
+			write (fd, buf, snprintf (buf, sizeof (buf),
+											  "sound=%s\n\n", sound_files[i]));
+		}
 	}
 
 	close (fd);
