@@ -23,11 +23,11 @@
 #ifdef USE_DEBUG
 #define malloc(n) xchat_malloc(n, __FILE__, __LINE__)
 #define realloc(n, m) xchat_realloc(n, m, __FILE__, __LINE__)
-#define free(n) xchat_free(n, __FILE__, __LINE__)
+#define free(n) xchat_dfree(n, __FILE__, __LINE__)
 #define strdup(n) xchat_strdup(n, __FILE__, __LINE__)
 void *xchat_malloc (int size, char *file, int line);
 void *xchat_strdup (char *str, char *file, int line);
-void xchat_free (void *buf, char *file, int line);
+void xchat_dfree (void *buf, char *file, int line);
 void *xchat_realloc (char *old, int len, char *file, int line);
 #endif
 
@@ -71,7 +71,7 @@ void *xchat_realloc (char *old, int len, char *file, int line);
 #define PATHLEN		255
 #define DOMAINLEN	100
 #define NICKLEN		64				/* including the NULL, so 63 really */
-#define CHANLEN		202
+#define CHANLEN		300
 #define PDIWORDS		32
 
 #define safe_strcpy(dest,src,len)	{strncpy(dest,src,len); \
@@ -130,6 +130,8 @@ struct xchatprefs
 	char logmask[256];
 	char stamp_format[64];
 	char timestamp_log_format[64];
+	char irc_id_ytext[64];
+	char irc_id_ntext[64];
 
 	char proxy_host[64];
 	int proxy_port;
@@ -177,6 +179,7 @@ struct xchatprefs
 	unsigned long dcc_ip;
 	char dcc_ip_str[DOMAINLEN + 1];
 
+	unsigned int tab_small;
 	unsigned int tab_dnd;
 	unsigned int tab_sort;
 	unsigned int mainwindow_save;
@@ -221,6 +224,7 @@ struct xchatprefs
 	unsigned int beepmsg;
 	unsigned int beepchans;
 	unsigned int beephilight;
+	unsigned int flash_hilight;
 	unsigned int truncchans;
 	unsigned int privmsgtab;
 	unsigned int logging;
@@ -234,6 +238,7 @@ struct xchatprefs
 	unsigned int show_away_once;
 	unsigned int show_away_message;
 	unsigned int auto_unmark_away;
+	unsigned int away_track;
 	unsigned int userhost;
 	unsigned int use_server_tab;
 	unsigned int notices_tabs;
@@ -267,6 +272,11 @@ struct xchatprefs
 
 	unsigned int msg_number_limit;	/*same deal */
 	unsigned int msg_time_limit;
+
+	/* Tells us if we need to save, only when they've been edited.
+		This is so that we continue using internal defaults (which can
+		change in the next release) until the user edits them. */
+	unsigned int save_pevents:1;
 };
 
 /* Session types */
@@ -283,8 +293,8 @@ typedef struct session
 	void *usertree;					/* ordered with Ops first */
 	struct User *me;					/* points to myself in the usertree */
 	char channel[CHANLEN];
-	char waitchannel[CHANLEN];		  /* waiting to join this channel */
-	char willjoinchannel[CHANLEN];	  /* /join done for this channel */
+	char waitchannel[CHANLEN];		  /* waiting to join channel (/join sent) */
+	char willjoinchannel[CHANLEN];	  /* will issue /join for this channel */
 	char channelkey[64];			  /* XXX correct max length? */
 	int limit;						  /* channel user limit */
 	int logfd;
@@ -344,6 +354,7 @@ typedef struct server
 	void (*p_quit)(struct server *, char *reason);
 	void (*p_kick)(struct server *, char *channel, char *nick, char *reason);
 	void (*p_part)(struct server *, char *channel, char *reason);
+	void (*p_nickserv)(struct server *, char *pass);
 	void (*p_join)(struct server *, char *channel, char *key);
 	void (*p_login)(struct server *, char *user, char *realname);
 	void (*p_join_info)(struct server *, char *channel);
@@ -389,10 +400,7 @@ typedef struct server
 	char *last_away_reason;
 	int pos;								/* current position in linebuf */
 	int nickcount;
-
-	char *username;
-	char *realname;
-	char *networkname;
+	int nickservtype;					/* 0=/MSG nickserv 1=/NICKSERV 2=/NS */
 
 	char *chantypes;					/* for 005 numeric - free me */
 	char *chanmodes;					/* for 005 numeric - free me */
@@ -401,8 +409,7 @@ typedef struct server
 	char *bad_nick_prefixes;		/* for ircd that doesn't give the modes */
 	int modes_per_line;				/* 6 on undernet, 4 on efnet etc... */
 
-	char *eom_cmd;						/* end-of-motd command, free it! */
-	void *network;						/* for servlistgui.c */
+	void *network;						/* points to entry in servlist.c or NULL! */
 
 	GSList *outbound_queue;
 	time_t next_send;						/* cptr->since in ircu */
@@ -443,6 +450,9 @@ typedef struct server
 	int supports_watch:1;		/* supports the WATCH command */
 	int bad_prefix:1;				/* gave us a bad PREFIX= 005 number */
 	unsigned int have_whox:1;	/* have undernet's WHOX features */
+	unsigned int have_capab:1;	/* supports CAPAB (005 tells us) */
+	unsigned int have_idmsg:1;	/* freenode's IDENTIFY-MSG */
+	unsigned int have_except:1;	/* ban exemptions +e */
 	int use_who:1;				/* whether to use WHO command to get dcc_ip */
 #ifdef USE_OPENSSL
 	int use_ssl:1;					  /* is server SSL capable? */
