@@ -421,7 +421,7 @@ mg_focus (session *sess)
 	/* dirty trick to avoid auto-selection */
 	GTK_ENTRY (sess->gui->input_box)->editable = 0;
 	gtk_widget_grab_focus (sess->gui->input_box);
-	GTK_ENTRY (sess->gui->input_box)->editable = 1;
+	gtk_editable_set_editable (GTK_EDITABLE (sess->gui->input_box), TRUE);
 
 	sess->server->front_session = sess;
 
@@ -634,6 +634,8 @@ mg_populate (session *sess)
 		gtk_widget_hide (gui->topicbutton_box);
 		/* hide the userlist */
 		mg_userlist_showhide (sess, FALSE);
+		/* shouldn't edit the topic */
+		gtk_editable_set_editable (GTK_EDITABLE (gui->topic_entry), FALSE);
 		break;
 	case SESS_SERVER:
 		if (prefs.chanmodebuttons)
@@ -642,6 +644,8 @@ mg_populate (session *sess)
 		gtk_widget_hide (gui->dialogbutton_box);
 		/* hide the userlist */
 		mg_userlist_showhide (sess, FALSE);
+		/* shouldn't edit the topic */
+		gtk_editable_set_editable (GTK_EDITABLE (gui->topic_entry), FALSE);
 		break;
 	default:
 		/* hide the dialog buttons */
@@ -650,6 +654,8 @@ mg_populate (session *sess)
 			gtk_widget_show (gui->topicbutton_box);
 		/* show the userlist */
 		mg_userlist_showhide (sess, TRUE);
+		/* let the topic be editted */
+		gtk_editable_set_editable (GTK_EDITABLE (gui->topic_entry), TRUE);
 	}
 
 	/* move to THE irc tab */
@@ -1240,7 +1246,7 @@ mg_topic_cb (GtkWidget *entry, gpointer userdata)
 	session *sess = current_sess;
 	char *text;
 
-	if (sess->channel[0] && sess->server->connected)
+	if (sess->channel[0] && sess->server->connected && sess->type == SESS_CHANNEL)
 	{
 		text = GTK_ENTRY (entry)->text;
 		if (text[0] == 0)
@@ -1633,7 +1639,7 @@ mg_dialog_button_cb (GtkWidget *wid, char *cmd)
 	/* dirty trick to avoid auto-selection */
 	GTK_ENTRY (current_sess->gui->input_box)->editable = 0;
 	gtk_widget_grab_focus (current_sess->gui->input_box);
-	GTK_ENTRY (current_sess->gui->input_box)->editable = 1;
+	gtk_editable_set_editable (GTK_EDITABLE (current_sess->gui->input_box), TRUE);
 }
 
 static void
@@ -1733,7 +1739,7 @@ mg_word_clicked (GtkWidget *xtext, char *word, GdkEventButton *even,
 			{
 			case WORD_URL:
 			case WORD_HOST:
-				goto_url (word);
+				fe_open_url (word);
 			}
 		}
 		return;
