@@ -96,41 +96,6 @@ struct mymenu
 };
 
 
-void
-goto_url (char *url)
-{
-#ifdef USE_GNOME
-	gnome_url_show (url);
-#else
-#ifdef WIN32
-	ShellExecute (0, "open", url, NULL, NULL, SW_SHOWNORMAL);
-#else
-	char tbuf[512], *moz;
-
-	/* gnome 2.4+ has this */
-	moz = g_find_program_in_path ("gnome-open");
-	if (moz)
-	{
-		snprintf (tbuf, sizeof (tbuf), "%s %s", moz, url);
-		g_free (moz);
-		xchat_exec (tbuf);
-		return;
-	}
-
-	moz = g_find_program_in_path ("gnome-moz-remote");
-	if (moz)
-	{
-		snprintf (tbuf, sizeof (tbuf), "%s %s", moz, url);
-		g_free (moz);
-	} else
-	{
-		snprintf (tbuf, sizeof (tbuf), "mozilla -remote 'openURL(%s)'", url);
-	}
-	xchat_exec (tbuf);
-#endif
-#endif
-}
-
 /* execute a userlistbutton/popupmenu command */
 
 static void
@@ -567,17 +532,20 @@ menu_showhide (void)
 	}
 }
 
+void
+menu_bar_toggle (void)
+{
+	prefs.hidemenu = !prefs.hidemenu;
+	menu_showhide ();
+}
+
 static void
 menu_middle_cb (GtkWidget *item, gpointer userdata)
 {
 	switch (GPOINTER_TO_INT (userdata))
 	{
 	case 0:
-		if (prefs.hidemenu)
-			prefs.hidemenu = 0;
-		else
-			prefs.hidemenu = 1;
-		menu_showhide ();
+		menu_bar_toggle ();
 		break;
 	case 1:
 		mg_topic_showhide (current_sess);
@@ -634,7 +602,7 @@ menu_middlemenu (session *sess, GdkEventButton *event)
 static void
 open_url_cb (GtkWidget *item, char *url)
 {
-	goto_url (url);
+	fe_open_url (url);
 }
 #endif
 
@@ -844,7 +812,7 @@ menu_newserver_window (GtkWidget * wid, gpointer none)
 	int old = prefs.tabchannels;
 
 	prefs.tabchannels = 0;
-	new_ircwindow (NULL, NULL, SESS_SERVER);
+	new_ircwindow (NULL, NULL, SESS_SERVER, 0);
 	prefs.tabchannels = old;
 }
 
@@ -854,7 +822,7 @@ menu_newchannel_window (GtkWidget * wid, gpointer none)
 	int old = prefs.tabchannels;
 
 	prefs.tabchannels = 0;
-	new_ircwindow (current_sess->server, NULL, SESS_CHANNEL);
+	new_ircwindow (current_sess->server, NULL, SESS_CHANNEL, 0);
 	prefs.tabchannels = old;
 }
 
@@ -862,10 +830,15 @@ static void
 menu_newserver_tab (GtkWidget * wid, gpointer none)
 {
 	int old = prefs.tabchannels;
+	int oldf = prefs.newtabstofront;
 
 	prefs.tabchannels = 1;
-	new_ircwindow (NULL, NULL, SESS_SERVER);
+	/* force focus if setting is "only requested tabs" */
+	if (prefs.newtabstofront == 2)
+		prefs.newtabstofront = 1;
+	new_ircwindow (NULL, NULL, SESS_SERVER, 0);
 	prefs.tabchannels = old;
+	prefs.newtabstofront = oldf;
 }
 
 static void
@@ -874,7 +847,7 @@ menu_newchannel_tab (GtkWidget * wid, gpointer none)
 	int old = prefs.tabchannels;
 
 	prefs.tabchannels = 1;
-	new_ircwindow (current_sess->server, NULL, SESS_CHANNEL);
+	new_ircwindow (current_sess->server, NULL, SESS_CHANNEL, 0);
 	prefs.tabchannels = old;
 }
 
@@ -1224,13 +1197,13 @@ menu_saveexit (GtkWidget *wid, gpointer none)
 static void
 menu_docs (GtkWidget *wid, gpointer none)
 {
-	goto_url ("http://xchat.org/docs/");
+	fe_open_url ("http://xchat.org/docs/");
 }
 
 /*static void
 menu_webpage (GtkWidget *wid, gpointer none)
 {
-	goto_url ("http://xchat.org");
+	fe_open_url ("http://xchat.org");
 }*/
 
 static void
@@ -1302,11 +1275,11 @@ static struct mymenu mymenu[] = {
 	{N_("S_ettings"), 0, 0, M_NEWMENU, 0, 1},	/* 33 */
 	{N_("Preferences..."), menu_settings, GTK_STOCK_PREFERENCES, M_MENUSTOCK, 0, 1},
 
-	{N_("Lists"), 0, GTK_STOCK_JUSTIFY_LEFT, M_MENUSUB, 0, 1},
+	{N_("Advanced"), 0, GTK_STOCK_JUSTIFY_LEFT, M_MENUSUB, 0, 1},
 	{N_("Auto Replace..."), menu_rpopup, 0, M_MENU, 0, 1},
 	{N_("CTCP Replies..."), menu_ctcpguiopen, 0, M_MENU, 0, 1},
 	{N_("Dialog Buttons..."), menu_dlgbuttons, 0, M_MENU, 0, 1},
-	{N_("Key Bindings..."), menu_keypopup, 0, M_MENU, 0, 1},
+	{N_("Keyboard Shortcuts..."), menu_keypopup, 0, M_MENU, 0, 1},
 	{N_("Text Events..."), menu_evtpopup, 0, M_MENU, 0, 1},
 	{N_("URL Handlers..."), menu_urlhandlers, 0, M_MENU, 0, 1},
 	{N_("User Commands..."), menu_usercommands, 0, M_MENU, 0, 1},
@@ -1342,7 +1315,11 @@ static struct mymenu mymenu[] = {
 
 	{N_("_Help"), 0, 0, M_NEWMENU, 0, 1},	/* 62 */
 	{N_("_Contents"), menu_docs, GTK_STOCK_HELP, M_MENUSTOCK, 0, 1, GDK_F1},
+#ifdef GTK_STOCK_ABOUT
+	{N_("_About"), menu_about, GTK_STOCK_ABOUT, M_MENUSTOCK, 0, 1},
+#else
 	{N_("_About"), menu_about, (char *)&pix_about, M_MENUPIX, 0, 1},
+#endif
 
 	{0, 0, 0, M_END, 0, 0},
 };
